#!/usr/bin/env python3 -u
# Copyright (c) Facebook, Inc. and its affiliates.
#
# This source code is licensed under the MIT license found in the
# LICENSE file in the root directory of this source tree.

"""
Translate pre-processed data with a trained model.
"""

from collections import defaultdict

import torch

from fairseq import checkpoint_utils, options, progress_bar, tasks, utils
from fairseq.meters import StopwatchMeter, TimeMeter
from fairseq.models import FairseqEncoder, FairseqDecoder, FairseqEncoderDecoderModel


###################################################################
##  PRISM PARAPHRASER GENERATION CODE
##
##  Here we build a fairseq model that gets ensembled with the regular multilngual NMT model
##  The new model simply downweights any vocabulary item that begins a new word that
##     matches an n-gram in the input (case is ignored).
##
##  See https://arxiv.org/abs/2008.04935 for more details on the generation modification
##  See https://arxiv.org/abs/2004.14564 for information about the model we use


def make_subword_penalties(line):  #TODO: Function not used?
    """
    prefix: n-grams of subwords (n=1,2,3,4)
    penalize: the next subword
    """
    penalties = defaultdict(list)
    toks = line.replace('<pad>', '').split()
    for prefix_len in (0,1,2,3):
        for ii in range(len(toks)-prefix_len):
            prefix = toks[ii:ii+prefix_len]
            next_word = toks[ii+prefix_len]
            penalties[tuple(prefix)].append( (next_word, len(prefix)) )

    return penalties

class NgramDownweightEncoder(FairseqEncoder):

    def __init__(self, args, dictionary):
        super().__init__(dictionary)
        self.mapx = self.make_vocab_start_map(self.dictionary.symbols)
        self.vocab_set = set(self.dictionary.symbols)
        self.args = args

    def make_word_penalties(self, line, vocab, mapx):
        """
        prefix: subwords that make up a n-gram (n=1,2,3,4) of FULL WORDS
        penalize: the next subword
        """
        penalties = defaultdict(list)
        uline = '▁'
        def breakup(tt):
            out = []
            for word in tt:
                for ii, subword in enumerate(word):
                    if ii == 0:
                        out.append(uline + subword)
                    else:
                        out.append(subword)
            return out
        line2 = line.replace('<pad>', '').strip()
        line2 = [x.replace('|', ' ').strip().split()
                 for x in line2.replace(' ', '|').split(uline) if x]

        for prefix_len in (0,1,2,3):
            for ii in range(len(line2)-prefix_len):
                prefix = line2[ii:ii+prefix_len]
                # just penalize starting a word, not continuing it
                next_word = uline + line2[ii+prefix_len][0]

                whole_next_word = uline + ''.join(line2[ii+prefix_len])

                word_prefix = breakup(prefix)

                # penalize any token that starts the next word, ignoring case
                # about 1s per line
                for tok in vocab:
                    if whole_next_word.lower().startswith(tok.lower()):
                        penalties[tuple(word_prefix)].append( (tok, len(prefix)) )

                # build the longest part of the next word I can that is in the vocab
                longest_next_substring = uline
                for subthing in line2[ii+prefix_len]:
                    if longest_next_substring+subthing in vocab:
                        longest_next_substring = longest_next_substring+subthing
                    else:
                        break

                for tok in mapx[longest_next_substring]:
                    # every word that starts the same, sans case
                    penalties[tuple(word_prefix)].append( (tok, len(prefix)) )

        return penalties

    def make_vocab_start_map(self, vocab):
        vocab_set = set(vocab)
        # build mapping from every lowercase subword to every cased
        # variant in vocabulary
        ucase2case = defaultdict(set)
        for word in vocab:
            for ii in range(1,len(word)+1):
                subword = word[:ii]
                if subword in vocab_set:
                    ucase2case[subword.lower()].add(subword)
        # build mapping from every word to every prefix that starts
        # that word (where "starts" ignores case)
        mapx = dict()
        for word in vocab:
            toks = set()
            for ii in range(1,len(word)+1):
                subword = word[:ii]
                for fubar in ucase2case[subword.lower()]:
                    toks.add(fubar)
            mapx[word] = list(toks)

        return mapx

    def forward(self, src_tokens, src_lengths):
        # Task, and in particular the ``'net_input'`` key in each
        # mini-batch. We discuss Tasks in the next tutorial, but for now just
        # know that *src_tokens* has shape `(batch, src_len)` and *src_lengths*
        # has shape `(batch)`.

        # Note that the source is typically padded on the left. This can be
        # configured by adding the `--left-pad-source "False"` command-line
        # argument, but here we'll make the Encoder handle either kind of
        # padding by converting everything to be right-padded.
        if self.args.left_pad_source:
            # Convert left-padding to right-padding.
            src_tokens = utils.convert_padding_direction(
                src_tokens,
                padding_idx=self.dictionary.pad(),
                left_to_right=True
            )

        # Return the Encoder's output. This can be any object and will be
        # passed directly to the Decoder.
        batch_size = src_tokens.shape[0]
        debug_out = self.dictionary.string(src_tokens,
                                           bpe_symbol=None, escape_unk=False)

        batch_penalties = []
        for line in debug_out.split('\n'):
            penalties = self.make_word_penalties(line=line, vocab=self.vocab_set, mapx=self.mapx)
            batch_penalties.append(penalties)

        return batch_penalties

    # Encoders are required to implement this method so that we can rearrange
    # the order of the batch elements during inference (e.g., beam search).
    def reorder_encoder_out(self, encoder_out, new_order):
        """
        Reorder encoder output according to `new_order`.

        Args:
            encoder_out: output from the ``forward()`` method
            new_order (LongTensor): desired order

        Returns:
            `encoder_out` rearranged according to `new_order`
        """
        # these maps arent going to be modified, so multiple references is fine
        return [encoder_out[ii] for ii in new_order.cpu().numpy()]


class NgramDownweightDecoder(FairseqDecoder):

    def __init__(self, args, dictionary):
        super().__init__(dictionary)
        self.args = args

    # During training Decoders are expected to take the entire target sequence
    # (shifted right by one position) and produce logits over the vocabulary.
    # The *prev_output_tokens* tensor begins with the end-of-sentence symbol,
    # ``dictionary.eos()``, followed by the target sequence.
    def forward(self, prev_output_tokens, encoder_out):
        """
        Args:
            prev_output_tokens (LongTensor): previous decoder outputs of shape
                `(batch, tgt_len)`, for teacher forcing
            encoder_out (Tensor, optional): output from the encoder, used for
                encoder-side attention

        Returns:
            tuple:
                - the last decoder layer's output of shape
                  `(batch, tgt_len, vocab)`
                - the last decoder layer's attention weights of shape
                  `(batch, tgt_len, src_len)`
        """

        batch_size = prev_output_tokens.shape[0]
        tgt_len = prev_output_tokens.shape[1]
        vocab_size = len(self.dictionary)

        xx = torch.zeros([batch_size, tgt_len, vocab_size], dtype=torch.float32)

        debug_out = self.dictionary.string(prev_output_tokens,
                                           bpe_symbol=None, escape_unk=False)

        lines = debug_out.split('\n')

        for ii, (line, penalties) in enumerate(zip(lines, encoder_out)):

            max_prefix_len = max([len(prefix) for prefix in penalties])

            #if ii == 0:
            #    print('#'*50)
            #    print('DECODE SO FAR:', line)

            toks = line.strip().split()
            #if ii == 0: print('input length:', len(toks))
            for n_gram_n in range(1, min(len(toks)+2, max_prefix_len)):
                prefix_size = n_gram_n - 1

                if prefix_size == 0:
                    prefix = tuple()
                else:
                    prefix = tuple(toks[-prefix_size:])

                if prefix in penalties:
                    for next_word, prefix_len_in_words in penalties[prefix]:
                        word_idx = self.dictionary.index(next_word)
                        xx[ii, -1, word_idx] -= self.args.prism_a * (prefix_len_in_words+1)  ** self.args.prism_b

        # Return the logits and ``None`` for the attention weights
        xx = xx.cuda()
        return xx, None


class NgramDownweightModel(FairseqEncoderDecoderModel):

    def max_positions(self):
        return (123456, 123456)

    @classmethod
    def build_model(cls, args, task):
        # Initialize our Encoder and Decoder.
        encoder = NgramDownweightEncoder(args=args, dictionary=task.source_dictionary)
        decoder = NgramDownweightDecoder(args=args, dictionary=task.target_dictionary)
        model = NgramDownweightModel(encoder, decoder)
        # Print the model architecture.
        #print(model)
        return model
    def get_normalized_probs(self, decoder_out, log_probs):
        return decoder_out[0]

class PrismParaphase:
    def __init__(self, a=0.003, b=4.0, *args, **kwargs):
        pass
    def generate(self, )


def main(args):
    assert args.path is not None, '--path required for generation!'
    assert not args.sampling or args.nbest == args.beam, \
        '--sampling requires --nbest to be equal to --beam'
    assert args.replace_unk is None or args.raw_text, \
        '--replace-unk requires a raw text dataset (--raw-text)'

    utils.import_user_module(args)

    if args.max_tokens is None and args.max_sentences is None:
        args.max_tokens = 12000
    print(args)




def cli_main():
    # Fairseq's options.
    parser = options.get_generation_parser()
    # add options for Prism paraphrase generation
    parser.add_argument('--prism_a', type=float, help='prism_a ** prism_b. reasonable starting point: 0.003', required=True)
    parser.add_argument('--prism_b', type=float, help='prism_a ** prism_b. reasonable starting point: 4.0', required=True)
    args = options.parse_args_and_arch(parser)
    main(args)


if __name__ == '__main__':
    cli_main()




import argparse
import hashlib
import logging
import os
import sys
from typing import List, Dict, Iterator, Any, Tuple

import numpy as np
import sentencepiece as spm
import torch
from fairseq import checkpoint_utils, utils
from fairseq.data import LanguagePairDataset


class SequenceScorer(object):
    """
    Copy of https://github.com/pytorch/fairseq/blob/master/fairseq/sequence_scorer.py
    with softmax temperature control added

    MIT License

    Copyright (c) Facebook, Inc. and its affiliates.

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in all
    copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
    AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
    LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
    OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
    SOFTWARE.
    """

    def __init__(self, tgt_dict, softmax_batch=None, temperature=1.0):
        self.pad = tgt_dict.pad()
        self.eos = tgt_dict.eos()
        self.softmax_batch = softmax_batch or sys.maxsize
        self.temperature = temperature
        assert self.softmax_batch > 0

    @torch.no_grad()
    def generate(self, models, sample, **kwargs):
        """Score a batch of translations."""
        net_input = sample['net_input']

        def batch_for_softmax(dec_out, target):
            # assumes decoder_out[0] is the only thing needed (may not be correct for future models!)
            first, rest = dec_out[0], dec_out[1:]
            bsz, tsz, dim = first.shape
            if bsz * tsz < self.softmax_batch:
                yield dec_out, target, True
            else:
                flat = first.contiguous().view(1, -1, dim)
                flat_tgt = target.contiguous().view(flat.shape[:-1])
                s = 0
                while s < flat.size(1):
                    e = s + self.softmax_batch
                    yield (flat[:, s:e],) + rest, flat_tgt[:, s:e], False
                    s = e

        def gather_target_probs(probs, target):
            probs = probs.gather(
                dim=2,
                index=target.unsqueeze(-1),
            )
            return probs

        orig_target = sample['target']

        # compute scores for each model in the ensemble
        avg_probs = None
        avg_attn = None
        for model in models:
            model.eval()
            decoder_out = model.forward(**net_input)
            attn = decoder_out[1]
            if type(attn) is dict:
                attn = attn.get('attn', None)

            batched = batch_for_softmax(decoder_out, orig_target)
            probs, idx = None, 0
            for bd, tgt, is_single in batched:
                sample['target'] = tgt

                # divide the logits by temperature prior to softmax
                # for example, see https://github.com/pytorch/fairseq/blob/master/fairseq/sequence_generator.py:
                #   decoder_out[0][:, -1:, :].div_(temperature)
                bd[0].div_(self.temperature)

                curr_prob = model.get_normalized_probs(bd, log_probs=len(models) == 1, sample=sample).data
                if is_single:
                    probs = gather_target_probs(curr_prob, orig_target)
                else:
                    if probs is None:
                        probs = curr_prob.new(orig_target.numel())
                    step = curr_prob.size(0) * curr_prob.size(1)
                    end = step + idx
                    tgt_probs = gather_target_probs(curr_prob.view(tgt.shape + (curr_prob.size(-1),)), tgt)
                    probs[idx:end] = tgt_probs.view(-1)
                    idx = end
                sample['target'] = orig_target

            probs = probs.view(sample['target'].shape)

            if avg_probs is None:
                avg_probs = probs
            else:
                avg_probs.add_(probs)
            if attn is not None and torch.is_tensor(attn):
                attn = attn.data
                if avg_attn is None:
                    avg_attn = attn
                else:
                    avg_attn.add_(attn)
        if len(models) > 1:
            avg_probs.div_(len(models))
            avg_probs.log_()
            if avg_attn is not None:
                avg_attn.div_(len(models))

        bsz = avg_probs.size(0)
        hypos = []
        start_idxs = sample['start_indices'] if 'start_indices' in sample else [0] * bsz
        for i in range(bsz):
            # remove padding from ref
            ref = utils.strip_pad(sample['target'][i, start_idxs[i]:], self.pad) \
                if sample['target'] is not None else None
            tgt_len = ref.numel()
            avg_probs_i = avg_probs[i][start_idxs[i]:start_idxs[i] + tgt_len]
            score_i = avg_probs_i.sum() / tgt_len
            if avg_attn is not None:
                avg_attn_i = avg_attn[i]
                alignment = utils.extract_hard_alignment(avg_attn_i, sample['net_input']['src_tokens'][i],
                                                         sample['target'][i], self.pad, self.eos)
            else:
                avg_attn_i = alignment = None
            hypos.append([{
                'tokens': ref,
                'score': score_i,
                'attention': avg_attn_i,
                'alignment': alignment,
                'positional_scores': avg_probs_i,
            }])
        return hypos



#!/usr/bin/env python3

import argparse

import os
import sys
import warnings
from typing import List, Dict, Iterator, Any, Tuple

import numpy as np
import sentencepiece as spm
import torch
from fairseq import checkpoint_utils, utils
from fairseq.data import LanguagePairDataset
from sacrebleu import get_source_file, get_reference_files, DATASETS, get_langpairs_for_testset

from prism.sequence_scorer import SequenceScorer
from prism.models import MODELS, hash_model, MODEL_DIR, PrismDataError

import logging
logger = logging.getLogger("prism")
logger.setLevel(logging.INFO)

class Prism:
    def __init__(self, lang, temperature=1.0):
        '''
        model_dir should contain:
         1) checkpoint.pt: the fairseq model
         2) spm.model: the sentencepiece model
         3) dict.src.txt: the fairseq source dictionary
         4) dict.tgt.txt: the fairseq target dictionary (likely a copy of the source)

        lang: ISO 639-1 Code (e.g. "en"). Must be a language compatable with the model.
        '''
        self.sp = spm.SentencePieceProcessor()
        self.sp.Load(MODEL_DIR + '/spm.model')

        self.lang = lang
        self.temperature = temperature

        # this prints things and I can't figure out how to disable it
        sys.stdout = open(os.devnull, 'w')
        self.models, self.args, self.task = checkpoint_utils.load_model_ensemble_and_task(
            [MODEL_DIR + '/checkpoint.pt', ],
            arg_overrides=dict(data=MODEL_DIR + '/'),
        )
        sys.stdout = sys.__stdout__

        self.use_cuda = torch.cuda.is_available()

        self.generator = SequenceScorer(self.task.target_dictionary, temperature=temperature)

        for model in self.models:
            if self.use_cuda:
                model.cuda()
            model.make_generation_fast_(
                beamable_mm_beam_size=None,
                need_attn=False,
            )
            #if fp16:
            #    model.half()

        # hash model
        self.model_hash = hash_model(MODEL_DIR)

        if self.model_hash in MODELS:
            model_langs = MODELS[self.model_hash]['langs']
            if lang not in model_langs:
                model_name = MODELS[self.model_hash]['name']
                raise PrismDataError(f'Language "{lang}" is unsupported for model "{model_name}"')
                raise PrismDataError(f'Supported languages for {model_name}: {", ".join(model_langs)}')
        else:
            raise PrismDataError('Unrecognized model, so cannot check language')

    def identifier(self):
        if self.model_hash in MODELS:
            model_name = MODELS[self.model_hash]['name']
        else:
            warnings.warn('unrecognized model, using hash to identify')
            model_name = self.model_hash

        return dict(version='0.1', model=model_name, seg_scores='avg_log_prob',
                    sys_scores='avg_log_prob', log_base=2, temperature=self.temperature)

    def _binarize(self, sentence: str) -> torch.LongTensor:
        return self.task.source_dictionary.encode_line(sentence, add_if_not_exist=False).long()

    def _encode(self, sent, prepend=True):
        sent = ' '.join(self.sp.EncodeAsPieces(sent))
        if prepend:
            sent = f'<{self.lang}> ' + sent
        return self._binarize(sent)

    def _build_batches(self,
                       source_tokens: List[List[int]],
                       target_tokens: List[List[int]],
                       skip_invalid_size_inputs: bool) -> Iterator[Dict[str, Any]]:
        source_lengths = torch.LongTensor([t.numel() for t in source_tokens])
        target_lengths = torch.LongTensor([t.numel() for t in target_tokens])

        batch_iterator = self.task.get_batch_iterator(
            dataset=LanguagePairDataset(source_tokens, source_lengths, self.task.source_dictionary,
                                        tgt=target_tokens, tgt_sizes=target_lengths,
                                        tgt_dict=self.task.target_dictionary),
            max_tokens=self.args.max_tokens,
            max_sentences=self.args.max_sentences,
            max_positions=(2000, 2000),  # ???
            ignore_invalid_inputs=skip_invalid_size_inputs,
        ).next_epoch_itr(shuffle=False)
        return batch_iterator

    def _score_forward(self, tok_sents_in, tok_sents_out):
        assert len(tok_sents_in) == len(tok_sents_out)
        tok_level_scores = [None, ] * len(tok_sents_in)  # for debug
        results = [None, ] * len(tok_sents_in)
        for batch in self._build_batches(tok_sents_in, tok_sents_out, skip_invalid_size_inputs=False):
            if self.use_cuda:  # must be a better way
                batch['id'] = batch['id'].cuda()
                batch['net_input']['src_tokens'] = batch['net_input']['src_tokens'].cuda()
                batch['net_input']['src_lengths'] = batch['net_input']['src_lengths'].cuda()
                batch['net_input']['prev_output_tokens'] = batch['net_input']['prev_output_tokens'].cuda()
                batch['target'] = batch['target'].cuda()

            translations = self.task.inference_step(self.generator, self.models, batch)

            ids = batch['id'].cpu().numpy()

            tok_scores = [x[0]['positional_scores'].cpu().numpy() for x in translations]

            # [1:] to skip language tag log prob
            sent_scores = [np.mean(x[1:]) for x in tok_scores]

            for _id, sent_score, _tok_score in zip(ids, sent_scores, tok_scores):
                results[_id] = sent_score
                tok_level_scores[_id] = _tok_score

        if logger.level == logging.DEBUG:
            for ii, (sent_in, scores_out, sent_out) in enumerate(zip(tok_sents_in, tok_level_scores, tok_sents_out)):
                sent_in_str = ' '.join([self.task.source_dictionary[x] for x in sent_in])
                logger.debug(f'Input[{ii}] = ' + sent_in_str)
                sent_out_tok = [self.task.source_dictionary[x] for x in sent_out]
                logger.debug(f'Output[{ii}] = ' + \
                             f' '.join([f'{a}[{b:.02f}]' for a, b in zip(sent_out_tok, scores_out)]))

        if None in results:
            raise Exception('Missing one or more sentence scores')

        return np.array(results)

    def score(self, cand, ref=None, src=None, segment_scores=False):

        if not (ref is None) ^ (src is None):
            raise Exception('Must provide exactly one of "ref" or "src"')

        tokenized_cand = [self._encode(sentence, prepend=False) for sentence in cand]
        tokenized_cand_prep = [self._encode(sentence, prepend=True) for sentence in cand]

        if src is not None:
            # Prism-src: score candidate given on source
            if len(cand) != len(src):
                raise Exception(f'Length of cand ({len(cand)}) does not match length of src ({len(src)})')
            tokenized_src = [self._encode(sentence, prepend=False) for sentence in src]
            scores = self._score_forward(tokenized_src, tokenized_cand_prep)

        else:
            # Prism-ref: average candidate given reference and reference given candidate
            if len(cand) != len(ref):
                raise Exception(f'Length of cand ({len(cand)}) does not match length of ref ({len(ref)})')
            tokenized_ref = [self._encode(sentence, prepend=False) for sentence in ref]
            tokenized_ref_prep = [self._encode(sentence, prepend=True) for sentence in ref]
            forward_scores = self._score_forward(tok_sents_in=tokenized_ref, tok_sents_out=tokenized_cand_prep)
            reverse_scores = self._score_forward(tok_sents_in=tokenized_cand, tok_sents_out=tokenized_ref_prep)
            scores = 0.5 * forward_scores + 0.5 * reverse_scores

        if not segment_scores:
            scores = np.mean(scores)

        return scores




import hashlib

import os

from torchnlp.download import download_file_maybe_extract

class PrismDataError(Exception):
    pass

def fetch_model():
    if os.environ.get("HOME", None):
        cache_directory = os.environ["HOME"] + "/.cache/prism-mt/"
        if not os.path.exists(cache_directory):
            os.makedirs(cache_directory)
            # Download data.
        if not os.path.exists(cache_directory+'/m39v1/'):
            download_file_maybe_extract(
                url="http://data.statmt.org/prism/m39v1.tar",
                directory=cache_directory,
            )
        return cache_directory
    else:
        raise Exception("HOME environment variable is not defined.")


MODELS = {
    '8412b2044da4b9b2c0a8ce87b305d0d1': {
        'name': 'm39v1',
        'path': 'todo',
        'date': '2020-04-30',
        'description': 'model released with arXiv paper April 2020',
        'langs': ['ar', 'bg', 'bn', 'ca', 'cs', 'da', 'de', 'el', 'en', 'es', 'et', 'eo', 'fi', 'fr', 'he',
                  'hr', 'hu', 'id', 'it', 'ja', 'kk', 'lt', 'lv', 'mk', 'nl', 'no', 'pl', 'pt', 'ro', 'ru',
                  'sk', 'sl', 'sq', 'sr', 'sv', 'tr', 'uk', 'vi', 'zh'],
    }
}

MODEL_DIR = os.environ["HOME"] + "/.cache/prism-mt/m39v1/"

def hash_model(model_dir):
    md5 = hashlib.md5()
    block_size = 2 ** 20
    for fname in ('checkpoint.pt', 'spm.model', 'dict.src.txt', 'dict.tgt.txt'):
        with open(os.path.join(model_dir, fname), "rb") as f:
            while True:
                data = f.read(block_size)
                if not data:
                    break
                md5.update(data)
    md5.digest()
    return md5.hexdigest()



from typing import List, Dict, Iterator, Any, Tuple

import logging
import sys

from sacrebleu import get_source_file, get_reference_files, DATASETS, get_langpairs_for_testset

class SacrebleuDataError(Exception):
    pass

def parse_sacrebleu_uri(uri: str) -> Tuple[str]:
    """
    Parses the test set and language pair from a URI of the form

        sacrebleu:wmt19:de-en
        sacrebleu:wmt19/google/ar:de-en
    """
    try:
        _, testset, langpair = uri.split(":")
    except ValueError:
        raise SacrebleuDataError('sacrebleu:* flags must take the form "sacrebleu:testset:langpair"')

    testsets = sorted(DATASETS, reverse=True)
    if testset not in testsets:
        raise SacrebleuDataError(f"Test set '{testset}' was not found. Available sacrebleu test sets are:")
        for key in testsets:
            raise SacrebleuDataError(f"  {key:20s}: {DATASETS[key].get('description', '')}")

    lang_pairs = get_langpairs_for_testset(testset)

    if langpair not in lang_pairs:
        raise SacrebleuDataError(
            f"Language pair '{langpair}' not available for testset '{testset}'.\n "
            f"Language pairs available for {testset}: {', '.join(lang_pairs)}")

    return testset, langpair



from setuptools import setup

setup(
  name = 'prism-mt',
  packages = ['prism'],
  version = '0.0.1',
  description = 'prism-mt',
  long_description = '',
  author = '',
  url = 'https://github.com/thompsonb/prism',
  keywords = [],
  install_requires = ['sentencepiece', 'fairseq', 'sacrebleu',
                      'torch', 'torchvision', 'pytorch-nlp'],
  classifiers = [
    "Programming Language :: Python :: 3",
    "Operating System :: OS Independent",
  ]
)
